//-----------------------------------------------------------------------------
// Title      : bsp_ub033_spi_slave
//-----------------------------------------------------------------------------
// Funktion   : SPI-Kommunikation zwischen einem Master und bis zu 8 Slaves
// Schaltung  : myAVR MK3
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 30.10.2011
// Version    : 1.10
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <UB_MK3_2560/MK3_2560_SPI_SLAVE_MSS.h>
#include <UB_MK3_2560/MK3_2560_LED.h>
#include <UB_MK3_2560/MK3_2560_ADC.h>

//-----------------------------------------------------------------------------
// Hinweis : 
// - Quick-Jumper Port-L muss gesetzt sein
// - Ein Poti muss an F0 angeschlossen sein
// - Diese Software initialisiert die CPU als Slave-1
// - SPI-Verbindung zum Master muss vorhanden sein (Multi-Slave-System)
//     Master-PB1 <-> Slave-PB1 (SPI SCK)
//     Master-PB2 <-> Slave-PB2 (SPI MOSI)
//     Master-PB3 <-> Slave-PB3 (SPI MISO)
//     Master-PE0 <-> Slave-PB0 (Slave-1 select)
// - die LEDs an Port-L wechseln dauernd
//   zwischen 0x0F und 0xF0 bis der Master
//   das erste Kommando an den Slave-1 gesendet hat
// - dann wird der Poti zyklisch ausgelesen
//   und im RAM an Adr.00h gespeichert
//   zustzlich wird der wert an den LEDs
//   ausgegeben
// - Der Master kann diesen Wert auslesen
// - Bei einem SPI-Fehler wird wieder auf den Master
//   gewartet usw.
//-----------------------------------------------------------------------------

//----------------------------------------------------
void wait4Master(void) {
  unsigned int n;
  SPI_KOM_STATUS_t check;
  char ready=0;
  
  while(ready==0) {
    // warte bis Master was gesendet hat
    
    for(n=0;n<50000;n++) {
      // kleine Pause
      MK3_LED_ZAHL(0x0F);
    }
    
    check=MK3_SPIS_AKTIVITY();
    if(check==KOM_CMD_EXE) {
      // master hat was gesendet
      ready=1;
    }
  
    for(n=0;n<50000;n++) {
      // kleine Pause
     MK3_LED_ZAHL(0xF0);
    }
  }
} 

//----------------------------------------------------
void handlePoti(void) {
  unsigned int uint;
  unsigned char wert;
  unsigned int n=0;
  char error=0;
  SPI_KOM_STATUS_t check;
  
  while(error==0) {
    // permanente Abfrage des Potis
    // bis ein SPI-Fehler erkannt wird
    
    // poti auslesen
    uint=MK3_ADC_READ_CH(ADC0);
    wert=(uint/4);
    
    // wert im RAM Adr.00 speichern
    MK3_SPIS_WR8_RAM(0x00,wert);
    
    for(n=0;n<1000;n++) {
      // kleine Pause und daten anzeigen
      MK3_LED_ZAHL(wert);
    }   
    
    // test ob ein SPI-Fehler vorliegt
    check=MK3_SPIS_AKTIVITY();
    if(check==KOM_CMD_ERR) {
      // fehler ist aufgetreten
      error=1;      
    }     
  }
}

//-----------------------------------------------------------------------------
// MAIN 
//-----------------------------------------------------------------------------
int main (void)
{
	char mode=0;
	
	MK3_INIT_LED();
	
	MK3_INIT_ADC();
	
	// init der CPU als SPI-SLAVE mit ID-Nr Slave1
	MK3_INIT_SPI_SLAVE(SPI_Slave1);
	
	while (true)
	{
		if(mode==0) {
			// warten auf Master
			wait4Master();
			// Master hat was gesendet
			mode=1;
		}
		else {
			// Poti bearbeiten
			handlePoti();
			// fehler beim SPI
			mode=0;
		}
	}
	return 0;
}
//-----------------------------------------------------------------------------






