//-----------------------------------------------------------------------------
// MK3_2560_SPI_MASTER_MSS
// Version 1.11
// 02.11.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board
// mit ATMega-2560 @ 16MHz
// SPI-Master-Funktion (Multi-Slave-System)
// SPI-Kommunikation zwischen einem Master und bis zu 8 Slaves
// Hinweis : es gibt auch einen Treiber fuer ein Single-Slave-System (SSS)
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_SPI_MASTER_MSS.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : Port-B (Bit0 bis Bit3) -> fuer die SPI-Pins
//          Port-E (komplett)      -> Select-Pins um 8 Slaves anzubinden
// Benutzt : MK3_2560_SPI_MSS.h
//-----------------------------------------------------------------------------
// Verbindungen :
// SCK vom Master muss zum SCK-Pin aller Slaves
// MOSI vom Master muss zum MOSI-Pin aller Slaves
// MISO vom Master muss zum MISO-Pin aller Slaves
// Pro Slave muss ein Select-Pin zum (passenden) SS-Pin vom Slave
//    (z.B. Select-2 muss zum SS-Pin von Slave-2)
//-----------------------------------------------------------------------------
#include  "UB_MK3_2560/MK3_2560_SPI_MSS.h"

//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_SPI_MASTER" starten
// 02. Funktion "MK3_SPIM_CHECK_SLAVE" starten um einen Slave zu suchen
//   -> Rueckgabewert muss >NO_FAILURE< sein
// 03. Funktion "MK3_SPIM_WRITE8_SLAVE" schreibt ein Byte ins RAM von einem Slave
// 04. Funktion "MK3_SPIM_READ8_SLAVE" liest ein Byte vom RAM eines Slaves
//  -> Rueckgabewert muss >NO_FAILURE< sein
//  04b. per Funktion "MK3_SPIM_RD8_DATA" das Daten Byte auslesen
// 05. Funktion "MK3_SPIM_WRITE16_SLAVE" schreibt ein INT ins RAM von einem Slave
// 06. Funktion "MK3_SPIM_READ16_SLAVE" liest ein INT vom RAM eines Slaves
//  -> Rueckgabewert muss >NO_FAILURE< sein
//  06b. per Funktion "MK3_SPIM_RD16_DATA" die INT Daten auslesen
// 07. Funktion "MK3_SPIM_RE_INIT_SLAVE" versucht den Slave neu zu initialisieren
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------



// Rueckgabewerte von "CHECK_SLAVE", "WRITE_SLAVE, "READ_SLAVE"
typedef enum {
  SPI_NO_FAILURE =0,    // Slave hat Kommando akzeptiert und bearbeitet
  SPI_ERR_NOT_DEFINED,  // error nr ist noch nicht definiert
  SPI_WRONG_DATA,       // uengueltige Daten
  SPI_ALL_DATA_LO,      // Alle empfangene Daten sind 0
  SPI_ALL_DATA_HI,      // Alle empfangene Daten sind 1
  SPI_WRONG_RX_XOR,     // XOR vom empfangenen Kommando falsch
  SPI_WRONG_RX_ID,      // ID vom empfangenen Kommando falsch
  SPI_UNKNOWN_RX,       // Unbekanntes Kommando empfangen
  SPI_NACK_ERROR,       // Slave meldet NACK (allgemein)
  SPI_NACK_XOR,         // Slave meldet XOR fehler
  SPI_NACK_ID,          // Slave meldet ID fehler
  SPI_NACK_POS,         // Slave meldet Positions fehler
  SPI_NACK_END,         // Slave meldet Ende-Kennungs fehler
  SPI_NACK_DATA,        // Slave meldet ungueltige Daten
  SPI_UNKNOWN_ERROR     // Slave meldet unbekanntes kommando (allgemein)
}SPI_SEND_ID_t;


//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_SPI_MASTER(void);
SPI_SEND_ID_t MK3_SPIM_CHECK_SLAVE(SPI_Slave_ID_t slave);
SPI_SEND_ID_t MK3_SPIM_WRITE8_SLAVE(SPI_Slave_ID_t slave, char adr, char daten);
SPI_SEND_ID_t MK3_SPIM_READ8_SLAVE(SPI_Slave_ID_t slave, char adr);
unsigned char MK3_SPIM_RD8_DATA(void);
SPI_SEND_ID_t MK3_SPIM_WRITE16_SLAVE(SPI_Slave_ID_t slave, char adr, unsigned int daten);
SPI_SEND_ID_t MK3_SPIM_READ16_SLAVE(SPI_Slave_ID_t slave, char adr);
unsigned int MK3_SPIM_RD16_DATA(void);
SPI_SEND_ID_t MK3_SPIM_RE_INIT_SLAVE(SPI_Slave_ID_t slave);
//-----------------------------------------------------------------------------



//----------------------------------------------------------------------------
// Definition der SPI-Pins vom Master
//----------------------------------------------------------------------------
#define MK3_DDR_SPI DDRB	// DDR-Register der SPI-Pins
#define MK3_PORT_SPI PORTB

#define MK3_SPI_SS 0x01         // SS von SPI (nc)
#define MK3_SPI_SCK 0x02    	// SCK von SPI
#define MK3_SPI_MOSI 0x04       // MOSI von SPI
#define MK3_SPI_MISO 0x08       // MISO von SPI


//----------------------------------------------------------------------------
// Definition der Select-Pins um die Slaves zu selectieren
// es koennen maximal 8 Slaves angeschlossen werden
//
// falls weniger Slaves benutzt werden und Port-Pins gespart werden muessen,
// einfach die nicht benutzten Pins ueberall auskommentieren 
//
// Sonderfall : wenn nur EIN einziger Slave benutzt wird,
// kann dieser auch am PB0 Pin vom Master angeschlossen werden
// dann ist der zusaetzliche Select-Port nicht notwendig 
// dafuer gibt es eigene Treiber -> Single-Slave-System
//----------------------------------------------------------------------------
#define MK3_DDR_SPI_SLAVE DDRE   // DDR-Register der SS-Leitungen
#define MK3_PORT_SPI_SLAVE PORTE // Port der SS-Leitungen

#define MK3_SPI_SLAVE1 0x01      // an diesem Pin liegt der SS vom Slave-1
#define MK3_SPI_SLAVE2 0x02      // an diesem Pin liegt der SS vom Slave-2
#define MK3_SPI_SLAVE3 0x04      // an diesem Pin liegt der SS vom Slave-3
#define MK3_SPI_SLAVE4 0x08      // an diesem Pin liegt der SS vom Slave-4
#define MK3_SPI_SLAVE5 0x10      // an diesem Pin liegt der SS vom Slave-5
#define MK3_SPI_SLAVE6 0x20      // an diesem Pin liegt der SS vom Slave-6
#define MK3_SPI_SLAVE7 0x40      // an diesem Pin liegt der SS vom Slave-7
#define MK3_SPI_SLAVE8 0x80      // an diesem Pin liegt der SS vom Slave-8


//----------------------------------------------------------------------------
// SPI-Clock einstellung
//----------------------------------------------------------------------------
#define SPCR_SPI_CLK  0x01 // @16MHz -> SPI_CLK = 1MHz


//-----------------------------------------------------------------------------
// Pausen Einstellungen
// damit keine Uebertragungsfehler durch zu schnelles senden entstehen
// diese Werte muessen eventuell an den langsamsten Slave angepasst werden
//-----------------------------------------------------------------------------

// Settings (wenn Master=16MHz und Slave=16MHz)
#define SPI_MASTER_PAUSE1  10  // kleine pause nach jedem Byte
#define SPI_MASTER_PAUSE2  150 // kleine pause nach jedem Kommando

// Settings (wenn Master=16MHz und Slave=8MHz)
//#define SPI_MASTER_PAUSE1  30  // kleine pause nach jedem Byte
//#define SPI_MASTER_PAUSE2  300 // kleine pause nach jedem Kommando


//-----------------------------------------------------------------------------
// interne Funktionen vom Master
//-----------------------------------------------------------------------------
SPI_B2_t SPIM_check_Rx_Command(unsigned char sid);
SPI_SEND_ID_t SPIM_check_Execute(SPI_B2_t err_akt);
void SPIM_makeMasterTx(char b0,char b1,char b2,char b3,char b4,char b5,char b6,char b7);
void SPIM_SEND_2_SLAVE(SPI_Slave_ID_t slave);
unsigned char SPIM_SEND_BYTE(unsigned char wert);
void SPIM_Select_Lo(SPI_Slave_ID_t slave);
void SPIM_Select_Hi(SPI_Slave_ID_t slave);



//-----------------------------------------------------------------------------
// interne Struktur vom Master
//-----------------------------------------------------------------------------
typedef struct {
  unsigned char slave_data_lo;  // hier stehen die uebergebene daten vom slave
  unsigned char slave_data_hi;  // hier stehen die uebergebene daten vom slave
}SPI_MASTER_t;
SPI_MASTER_t SPI_MASTER;


//----------------------------------------------------------------------
// Initialisiert die CPU als SPI-Master
// WICHTIG : es darf nur einen Master im System geben
//----------------------------------------------------------------------
void MK3_INIT_SPI_MASTER(void) {
  unsigned int n=0;

  #if SPI_DATA_BYTES != 10
    // WICHTIG !! Anzahl der DatenBytes MUSS 10 sein
    spi_forced_error=1; 
  #endif 
  
  // SPI-Pins definieren---------------------------
  MK3_DDR_SPI|=MK3_SPI_SS;     // SS als Ausgang (WICHTIG)
  MK3_PORT_SPI|=MK3_SPI_SS;    // Pin auf Hi
  MK3_DDR_SPI|=MK3_SPI_SCK;    // SCK als Ausgang
  MK3_DDR_SPI|=MK3_SPI_MOSI;   // MOSI als Ausgang
  MK3_DDR_SPI&=~MK3_SPI_MISO;  // MISO als Eingang
  MK3_PORT_SPI|=MK3_SPI_MISO;  // Pull-Up aktivieren
    
  // SS_Pins der Slaves definieren-----------------
  MK3_DDR_SPI_SLAVE|=MK3_SPI_SLAVE1;  // Pin auf Ausgang
  MK3_DDR_SPI_SLAVE|=MK3_SPI_SLAVE2;  // Pin auf Ausgang
  MK3_DDR_SPI_SLAVE|=MK3_SPI_SLAVE3;  // Pin auf Ausgang
  MK3_DDR_SPI_SLAVE|=MK3_SPI_SLAVE4;  // Pin auf Ausgang
  MK3_DDR_SPI_SLAVE|=MK3_SPI_SLAVE5;  // Pin auf Ausgang
  MK3_DDR_SPI_SLAVE|=MK3_SPI_SLAVE6;  // Pin auf Ausgang
  MK3_DDR_SPI_SLAVE|=MK3_SPI_SLAVE7;  // Pin auf Ausgang
  MK3_DDR_SPI_SLAVE|=MK3_SPI_SLAVE8;  // Pin auf Ausgang
  MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE1; // Pin auf Hi
  MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE2; // Pin auf Hi
  MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE3; // Pin auf Hi
  MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE4; // Pin auf Hi 
  MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE5; // Pin auf Hi 
  MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE6; // Pin auf Hi 
  MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE7; // Pin auf Hi 
  MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE8; // Pin auf Hi 
    
  // init der Struktur------------------------------
  for(n=0;n<SPI_DATA_BYTES;n++) {
    SPI_DATA.tx[n]=0x00; 
    SPI_DATA.rx[n]=0x00; 
  }
  
  // Reset aller Variablen---------------------------
  SPI_MASTER.slave_data_lo=0;
  SPI_MASTER.slave_data_hi=0;

  // aktivieren des SPI-Ports-------------------------
  SPCR=0x00;
  SPCR|=SPCR_SPI_CLK; // CLK = 1MHz
  SPCR|=0x10; // Master Mode
  SPCR|=0x40; // SPI aktivieren
  
}

//----------------------------------------------------------------------
// sucht einen Slave
// -> ruckgabe = ok, bzw. fehlermeldung
// -> in slave_data_lo steht bei ok das status_byte vom Slave
//----------------------------------------------------------------------
SPI_SEND_ID_t MK3_SPIM_CHECK_SLAVE(SPI_Slave_ID_t slave) {
  SPI_SEND_ID_t ret_wert=SPI_NO_FAILURE;
  SPI_B2_t slave_rx=RX_OK;
  unsigned char sid=0;    
  
  // SID bestimmen
  sid=SPIMS_getSID(slave);
  
  //----------------------------------------
  // Kommando-1 erstellen (sendet die daten)
  //----------------------------------------
  SPIM_makeMasterTx(sid,CMD_CHK_SLAVE,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
  
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);  
  
  //----------------------------------------
  // Kommando-2 erstellen (pollt das ergebnis)
  //----------------------------------------
  SPIM_makeMasterTx(sid,CMD_POLL_SLAVE,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
  
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);  
  
  //----------------------------------------
  // antwort vom slave pruefen
  //----------------------------------------
  slave_rx=SPIM_check_Rx_Command(sid);
  ret_wert=SPIM_check_Execute(slave_rx);
  
  if(ret_wert==SPI_NO_FAILURE) {
    // wenn keine fehler
    // daten auslesen [byte2+3] und speichern
    SPI_MASTER.slave_data_lo=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);
  }  

  return(ret_wert);
}

//----------------------------------------------------------------------
// sendet ein byte zum Slave zum speichern im RAM
// -> ruckgabe = ok, bzw. fehlermeldung
// -> in slave_data_lo steht bei ok die geschriebenen daten vom slave
//----------------------------------------------------------------------
SPI_SEND_ID_t MK3_SPIM_WRITE8_SLAVE(SPI_Slave_ID_t slave,char adr,char daten) {   
  SPI_SEND_ID_t ret_wert=SPI_NO_FAILURE;    
  SPI_B2_t slave_rx=RX_OK;
  unsigned char sid;  
  unsigned char ahi,alo,dhi,dlo;  
  
  // adresse pruefen
  if(adr<MAX_SPI_RAM) {

  }
  else {
    ret_wert=SPI_WRONG_DATA;
    return(ret_wert);    
  }
  
  // SID bestimmen
  sid=SPIMS_getSID(slave);  
  
  //----------------------------------------
  // Kommando-1 erstellen (sendet die daten)
  //----------------------------------------
  ahi=SPIMS_makeByteHi(0x00,adr);
  alo=SPIMS_makeByteLo(0x00,adr); 
  dhi=SPIMS_makeByteHi(0x00,daten);
  dlo=SPIMS_makeByteLo(0x00,daten);   
  SPIM_makeMasterTx(sid,CMD_WR8_RAM,ahi,alo,dhi,dlo,CMD_0,CMD_0);
  
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);
  
  //----------------------------------------
  // Kommando-2 erstellen (pollt das ergebnis)
  //----------------------------------------
  SPIM_makeMasterTx(sid,CMD_POLL_SLAVE,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
    
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);  
  
  //----------------------------------------
  // antwort vom slave pruefen
  //----------------------------------------
  slave_rx=SPIM_check_Rx_Command(sid);
  ret_wert=SPIM_check_Execute(slave_rx);
  
  if(ret_wert==SPI_NO_FAILURE) {
    // wenn keine fehler
    // daten auslesen [byte2+3] und speichern
    SPI_MASTER.slave_data_lo=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);
  }   

  return(ret_wert);  
}

//----------------------------------------------------------------------
// liest ein byte vom Slave aus dem RAM
// -> ruckgabe = ok, bzw. fehlermeldung
// -> in slave_data_lo steht bei ok die gelesenen daten vom slave
//----------------------------------------------------------------------
SPI_SEND_ID_t MK3_SPIM_READ8_SLAVE(SPI_Slave_ID_t slave, char adr) {
  SPI_SEND_ID_t ret_wert=SPI_NO_FAILURE;
  SPI_B2_t slave_rx=RX_OK;
  unsigned char sid; 
  unsigned char ahi,alo;
  
  // adresse pruefen
  if(adr<MAX_SPI_RAM) {

  }
  else {
    ret_wert=SPI_WRONG_DATA;
    return(ret_wert);    
  }  
  
  // SID bestimmen
  sid=SPIMS_getSID(slave);  
  
  //----------------------------------------
  // Kommando-1 erstellen (sendet die daten)
  //----------------------------------------
  ahi=SPIMS_makeByteHi(0x00,adr);
  alo=SPIMS_makeByteLo(0x00,adr);   
  SPIM_makeMasterTx(sid,CMD_RD8_RAM,ahi,alo,CMD_0,CMD_0,CMD_0,CMD_0);
  
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);   
  
  //----------------------------------------
  // Kommando-2 erstellen (pollt das ergebnis)
  //----------------------------------------
  SPIM_makeMasterTx(sid,CMD_POLL_SLAVE,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
  
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);  
  
  //----------------------------------------
  // antwort vom slave pruefen
  //----------------------------------------
  slave_rx=SPIM_check_Rx_Command(sid);
  ret_wert=SPIM_check_Execute(slave_rx);

  if(ret_wert==SPI_NO_FAILURE) {
    // wenn keine fehler
    // daten auslesen [byte2+3] und speichern
    SPI_MASTER.slave_data_lo=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);
  } 
   
  return(ret_wert);
}

//----------------------------------------------------------------------
// sendet ein integer zum Slave zum speichern im RAM
// -> ruckgabe = ok, bzw. fehlermeldung
// -> in slave_data_lo und hi steht bei ok die geschriebenen daten vom slave
//----------------------------------------------------------------------
SPI_SEND_ID_t MK3_SPIM_WRITE16_SLAVE(SPI_Slave_ID_t slave,char adr,unsigned int daten) {   
  SPI_SEND_ID_t ret_wert=SPI_NO_FAILURE;    
  SPI_B2_t slave_rx=RX_OK;
  unsigned char sid;  
  unsigned char hi,lo;
  unsigned char ahi,alo;
  unsigned char d0hi,d0lo,d1hi,d1lo;
  
  // adresse pruefen
  if(adr<MAX_SPI_RAM-1) {

  }
  else {
    ret_wert=SPI_WRONG_DATA;
    return(ret_wert);    
  }
 
  // Daten aufteilen
  lo=(unsigned char)(daten&0x00FF);    
  hi=(unsigned char)(daten>>8);
  
  // SID bestimmen
  sid=SPIMS_getSID(slave);  
  
  //----------------------------------------
  // Kommando-1 erstellen (sendet die daten)
  //----------------------------------------
  ahi=SPIMS_makeByteHi(0x00,adr);
  alo=SPIMS_makeByteLo(0x00,adr); 
  d1hi=SPIMS_makeByteHi(0x00,hi);
  d1lo=SPIMS_makeByteLo(0x00,hi); 
  d0hi=SPIMS_makeByteHi(0x00,lo);
  d0lo=SPIMS_makeByteLo(0x00,lo);   
  SPIM_makeMasterTx(sid,CMD_WR16_RAM,ahi,alo,d1hi,d1lo,d0hi,d0lo);
  
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);
    
  //----------------------------------------
  // Kommando-2 erstellen (pollt das ergebnis)
  //----------------------------------------
  SPIM_makeMasterTx(sid,CMD_POLL_SLAVE,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
    
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);  
  
  //----------------------------------------
  // antwort vom slave pruefen
  //----------------------------------------
  slave_rx=SPIM_check_Rx_Command(sid);
  ret_wert=SPIM_check_Execute(slave_rx);
  
  if(ret_wert==SPI_NO_FAILURE) {
    // wenn keine fehler
    // daten auslesen [byte2+3+4+5] und speichern
    SPI_MASTER.slave_data_hi=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);
    SPI_MASTER.slave_data_lo=SPIMS_getByte(SPI_DATA.rx[4],SPI_DATA.rx[5]);
  }   

  return(ret_wert);  
}

//----------------------------------------------------------------------
// liest ein Integer vom Slave aus dem RAM
// -> ruckgabe = ok, bzw. fehlermeldung
// -> in slave_data_lo und hi steht bei ok die gelesenen daten vom slave
//----------------------------------------------------------------------
SPI_SEND_ID_t MK3_SPIM_READ16_SLAVE(SPI_Slave_ID_t slave, char adr) {
  SPI_SEND_ID_t ret_wert=SPI_NO_FAILURE;
  SPI_B2_t slave_rx=RX_OK;
  unsigned char sid; 
  unsigned char ahi,alo;
  
  // adresse pruefen
  if(adr<MAX_SPI_RAM-1) {

  }
  else {
    ret_wert=SPI_WRONG_DATA;
    return(ret_wert);    
  }  
  
  // SID bestimmen
  sid=SPIMS_getSID(slave);  
  
  //----------------------------------------
  // Kommando-1 erstellen (sendet die daten)
  //----------------------------------------
  ahi=SPIMS_makeByteHi(0x00,adr);
  alo=SPIMS_makeByteLo(0x00,adr);   
  SPIM_makeMasterTx(sid,CMD_RD16_RAM,ahi,alo,CMD_0,CMD_0,CMD_0,CMD_0);
  
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);  
     
  //----------------------------------------
  // Kommando-2 erstellen (pollt das ergebnis)
  //----------------------------------------
  SPIM_makeMasterTx(sid,CMD_POLL_SLAVE,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
  
  // daten zum slave senden
  SPIM_SEND_2_SLAVE(slave);  
  
  //----------------------------------------
  // antwort vom slave pruefen
  //----------------------------------------
  slave_rx=SPIM_check_Rx_Command(sid);
  ret_wert=SPIM_check_Execute(slave_rx);

  if(ret_wert==SPI_NO_FAILURE) {
    // wenn keine fehler
    // daten auslesen [byte2+3+4+5] und speichern
    SPI_MASTER.slave_data_hi=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);
    SPI_MASTER.slave_data_lo=SPIMS_getByte(SPI_DATA.rx[4],SPI_DATA.rx[5]);
  } 
   
  return(ret_wert);
}


//----------------------------------------------------------------------
// versucht den Slave neu zu initialisieren
// -> ruckgabe = ok, bzw. fehlermeldung
// -> in slave_data_lo steht bei ok das status_byte vom Slave
//----------------------------------------------------------------------
SPI_SEND_ID_t MK3_SPIM_RE_INIT_SLAVE(SPI_Slave_ID_t slave) {
  SPI_SEND_ID_t ret_wert=SPI_NO_FAILURE;;
  unsigned char re_init;
  unsigned int pause=0;

  // SS_Pin vom Slave auf Lo-Pegel legen
  SPIM_Select_Lo(slave);

  // ein re-init byte per SPI senden
  re_init=SPIMS_makeByteLo(MSB_BYTE_9,0x00); // endekennung senden
  SPIM_SEND_BYTE(re_init);                   // byte senden 

  // SS_Pin vom Slave auf Hi-Pegel legen
  SPIM_Select_Hi(slave); 

  for(pause=0;pause<(SPI_MASTER_PAUSE1+SPI_MASTER_PAUSE2);pause++) {
    // kleine Pause (WICHTIG) 
	__asm volatile ("nop \n");  // mini Pause
  } 

  // Slave nochmal neu suchen
  ret_wert=MK3_SPIM_CHECK_SLAVE(slave);

  return(ret_wert);
}

//----------------------------------------------------------------------
// uebergibt ein Byte vom zuletzt ausgefuehrten Kommando
//----------------------------------------------------------------------
unsigned char MK3_SPIM_RD8_DATA(void) {
  unsigned char ret_wert=0;
  
  ret_wert=SPI_MASTER.slave_data_lo;
  
  return(ret_wert);
}

//----------------------------------------------------------------------
// uebergibt ein Integer vom zuletzt ausgefuehrten Kommando
//----------------------------------------------------------------------
unsigned int MK3_SPIM_RD16_DATA(void) {
  unsigned int ret_wert=0;
  unsigned char lo,hi;
  
  lo=SPI_MASTER.slave_data_lo;
  hi=SPI_MASTER.slave_data_hi;
  
  ret_wert=(hi<<8);
  ret_wert=(ret_wert&0xFF00);
  ret_wert+=lo;   
  
  return(ret_wert);
}

//----------------------------------------------------------------------
// Ueberprufen ob das gesendete Kommando
// vom Slave ausgefuehrt wurde
//----------------------------------------------------------------------
SPI_SEND_ID_t SPIM_check_Execute(SPI_B2_t err_akt) {
  SPI_SEND_ID_t ret_wert=SPI_NO_FAILURE;
  unsigned char cmd,wert;
  
  if(err_akt==RX_OK) {
    // wenn noch kein fehler festgestellt wurden 
    ret_wert=SPI_ERR_NOT_DEFINED;
    // pruefen ob der Slave ein ACK gesendet hat
    cmd=(SPI_DATA.rx[BPOS_CMD]&0x0F);
    if(cmd==(CMD_ACK&0x0F)) {
      // slave hat ein ACK gesendet
      ret_wert=SPI_NO_FAILURE;
    }
    else if(cmd==(CMD_NACK&0x0F)) {
      // slave hat ein NACK gesendet
      ret_wert=SPI_NACK_ERROR;
      // 2tes Byte auswerten
      wert=(SPI_DATA.rx[2]&0x0F);
      if(wert==(RX_WRONG_XOR&0x0F)) ret_wert=SPI_NACK_XOR;
      if(wert==(RX_WRONG_ID&0x0F)) ret_wert=SPI_NACK_ID;
      if(wert==(RX_WRONG_POS&0x0F)) ret_wert=SPI_NACK_POS;
      if(wert==(RX_WRONG_END&0x0F)) ret_wert=SPI_NACK_END;
      if(wert==(RX_WRONG_DATA&0x0F)) ret_wert=SPI_NACK_DATA;
    }
    else if(cmd==(CMD_UNKNOWN&0x0F)) {
      // slave hat ein unknown gemeldet
      ret_wert=SPI_UNKNOWN_ERROR;
    }
    else {
      // die Antwort vom Slave ist dem Master nicht bekannt
      ret_wert=SPI_UNKNOWN_RX;   
    }    
  }
  else {
    // wenn schon ein fehler festgestellt wurde
    ret_wert=SPI_ERR_NOT_DEFINED;
    if(err_akt==RX_ALL_DATA_0) ret_wert=SPI_ALL_DATA_LO;
    if(err_akt==RX_ALL_DATA_1) ret_wert=SPI_ALL_DATA_HI;
    if(err_akt==RX_WRONG_XOR) ret_wert=SPI_WRONG_RX_XOR;
    if(err_akt==RX_WRONG_ID) ret_wert=SPI_WRONG_RX_ID;         
  }
  
  return(ret_wert);
}

//----------------------------------------------------------------------
// Ueberprufen des empfangenen Kommandos auf Fehler
// in der Checksumme oder in der ID
// erkennt auch ob alle Bits Lo oder alle Bits Hi sind
//----------------------------------------------------------------------
SPI_B2_t SPIM_check_Rx_Command(unsigned char sid) {  
  SPI_B2_t ret_wert=RX_OK;
  unsigned char wert1=0,wert2=0;
  unsigned int n=0;
  unsigned int sum=0;
  
  // pruefen ob alle daten hi oder lo
  sum=0;
  for(n=0;n<SPI_DATA_BYTES;n++) {    
    sum+=SPI_DATA.rx[n];
  }
  if(sum==0) ret_wert=RX_ALL_DATA_0;
  if(sum==MAX_SPI_DATA_WERT) ret_wert=RX_ALL_DATA_1;  
  
  if(ret_wert==RX_OK) {
    // pruefen der empfangenen checksumme
    wert1=SPIMS_makeChecksum_RX();
    wert2=SPIMS_getByte(SPI_DATA.rx[BPOS_XOR_HI],SPI_DATA.rx[BPOS_XOR_LO]);
    if(wert1!=wert2) {
      ret_wert=RX_WRONG_XOR; // falsche XOR    
    }      
    else {
      // pruefen ob alle Bytes eine Slave-ID als MSB haben
      wert1=(MSB_SLAVE_ID&0xF0);
      for(n=0;n<SPI_DATA_BYTES;n++) {
        wert2=(SPI_DATA.rx[n]&0xF0);
        if(wert1!=wert2) ret_wert=RX_WRONG_ID; // falsche ID
      }
      
      if(ret_wert==RX_OK) {
        // pruefen auf die richtige Slave-ID 
        wert1=(sid&0x0F);
        wert2=SPI_DATA.rx[BPOS_ID]&0x0F;
        if(wert1!=wert2) ret_wert=RX_WRONG_ID; // falsche ID
      }
    }
  }
  
  return(ret_wert);
}

//----------------------------------------------------------------------
// Erstellt ein Kommando (bestehend aus 10Bytes)
// das dann gesendet werden kann
// -> 8 Bytes werden uebergeben, 2Bytes bilden die Checksumme
//----------------------------------------------------------------------
void SPIM_makeMasterTx(char b0, char b1, char b2, char b3, char b4, char b5, char b6, char b7) {
  unsigned char chksum=0;
  
  // TX-Data erstellen  
  SPI_DATA.tx[0]=SPIMS_makeByteLo(MSB_BYTE_0,b0);
  SPI_DATA.tx[1]=SPIMS_makeByteLo(MSB_BYTE_1,b1);
  SPI_DATA.tx[2]=SPIMS_makeByteLo(MSB_BYTE_2,b2);
  SPI_DATA.tx[3]=SPIMS_makeByteLo(MSB_BYTE_3,b3);
  SPI_DATA.tx[4]=SPIMS_makeByteLo(MSB_BYTE_4,b4);
  SPI_DATA.tx[5]=SPIMS_makeByteLo(MSB_BYTE_5,b5);
  SPI_DATA.tx[6]=SPIMS_makeByteLo(MSB_BYTE_6,b6);
  SPI_DATA.tx[7]=SPIMS_makeByteLo(MSB_BYTE_7,b7);
  
  // checksumme erstellen
  chksum=SPIMS_makeChecksum_TX();
  
  // checksumme einfuegen
  SPI_DATA.tx[8]=SPIMS_makeByteHi(MSB_BYTE_8,chksum);
  SPI_DATA.tx[9]=SPIMS_makeByteLo(MSB_BYTE_9,chksum);  
 
  // letztes byte = startbyte
  SPI_DATA.tx[10]=SPI_DATA.tx[0];
}


//----------------------------------------------------------------------
// sendet ein Komplettes Kommando (10 Bytes) an einen Slave
// und empfaengt im gegenzug ein Komplettes Kommando (10 Bytes) vom Slave
// das dauert im Moment (Version 1.8) ca. 310us
//----------------------------------------------------------------------
void SPIM_SEND_2_SLAVE(SPI_Slave_ID_t slave) {
  unsigned int n=0,pause=0;
  unsigned char wert=0;   
      
  for(n=0;n<SPI_DATA_BYTES;n++) {
    // SS_Pin vom Slave auf Lo-Pegel legen
    SPIM_Select_Lo(slave);
    
    // ein byte per SPI senden
    wert=SPIM_SEND_BYTE(SPI_DATA.tx[n]); // byte senden/empfangen
    SPI_DATA.rx[n]=wert;                 // rx-data speichern        
    
    // SS_Pin vom Slave auf Hi-Pegel legen
    SPIM_Select_Hi(slave);        
    
    for(pause=0;pause<SPI_MASTER_PAUSE1;pause++) {
      // kleine Pause (WICHTIG) 
      // um dem Slave nach jedem Byte etwas zeit zu geben
      // @16MHz 10 -> ca. 10us
      // @16MHz 30 -> ca. 20us
	  __asm volatile ("nop \n");  // mini Pause
    }    
  }   
  
  for(pause=0;pause<SPI_MASTER_PAUSE2;pause++) {
    // kleine Pause (WICHTIG) 
    // um dem Slave nach jedem Kommando etwas zeit zu geben
    // @16MHz 150 -> ca. 100us
    // @16MHz 300 -> ca. 200us
    
    // der Slave braucht ca. 70us Zeit fuer die Verarbeitung (@16MH)
    //                   ca  150us (@8MHz)
	__asm volatile ("nop \n");  // mini Pause
  }  
}

//----------------------------------------------------------------------
// Slave-Select-Pin vom Slave auf Lo-Pegel legen
//----------------------------------------------------------------------
void SPIM_Select_Lo(SPI_Slave_ID_t slave) {
  if(slave==SPI_Slave1) MK3_PORT_SPI_SLAVE&=~MK3_SPI_SLAVE1; // Slave1 auf Lo
  if(slave==SPI_Slave2) MK3_PORT_SPI_SLAVE&=~MK3_SPI_SLAVE2; // Slave2 auf Lo
  if(slave==SPI_Slave3) MK3_PORT_SPI_SLAVE&=~MK3_SPI_SLAVE3; // Slave3 auf Lo
  if(slave==SPI_Slave4) MK3_PORT_SPI_SLAVE&=~MK3_SPI_SLAVE4; // Slave4 auf Lo
  if(slave==SPI_Slave5) MK3_PORT_SPI_SLAVE&=~MK3_SPI_SLAVE5; // Slave5 auf Lo
  if(slave==SPI_Slave6) MK3_PORT_SPI_SLAVE&=~MK3_SPI_SLAVE6; // Slave6 auf Lo  
  if(slave==SPI_Slave7) MK3_PORT_SPI_SLAVE&=~MK3_SPI_SLAVE7; // Slave7 auf Lo 
  if(slave==SPI_Slave8) MK3_PORT_SPI_SLAVE&=~MK3_SPI_SLAVE8; // Slave8 auf Lo 
}

//----------------------------------------------------------------------
// Slave-Select-Pin vom Slave auf Hi-Pegel legen
//----------------------------------------------------------------------
void SPIM_Select_Hi(SPI_Slave_ID_t slave) {
  if(slave==SPI_Slave1) MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE1; // Slave1 auf Hi
  if(slave==SPI_Slave2) MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE2; // Slave2 auf Hi
  if(slave==SPI_Slave3) MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE3; // Slave3 auf Hi
  if(slave==SPI_Slave4) MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE4; // Slave4 auf Hi
  if(slave==SPI_Slave5) MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE5; // Slave5 auf Hi
  if(slave==SPI_Slave6) MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE6; // Slave6 auf Hi
  if(slave==SPI_Slave7) MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE7; // Slave7 auf Hi
  if(slave==SPI_Slave8) MK3_PORT_SPI_SLAVE|=MK3_SPI_SLAVE8; // Slave8 auf Hi
}

//----------------------------------------------------------------------
// sendet ein Byte per SPI an den Slave
// empfaengt gleichzeitig ein Byte vom Slave
//----------------------------------------------------------------------
unsigned char SPIM_SEND_BYTE(unsigned char wert) {
  unsigned char ret_wert=0;
  unsigned int n=0;
	
  SPDR = wert;           // sendet ein Byte

  for(n=0;n<5000;n++) {
    if((SPSR & 0x80)!=0) break;  // warte bis gesendet wurde
  }

  ret_wert=SPDR;         // empfaengt ein byte

  return(ret_wert);
}



