//-----------------------------------------------------------------------------
// Title      : bsp_ub034_spi_master
//-----------------------------------------------------------------------------
// Funktion   : SPI-Kommunikation zwischen einem Master und einem Slave
// Schaltung  : myAVR MK3
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 30.10.2011
// Version    : 1.10
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <UB_MK3_2560/MK3_2560_SPI_MASTER_SSS.h>
#include <UB_MK3_2560/MK3_2560_LED.h>
#include <UB_MK3_2560/MK3_2560_KEY.h>

//-----------------------------------------------------------------------------
// Hinweis : 
// - Quick-Jumper Port-L muss gesetzt sein
// - Quick-Jumper Port-K muss gesetzt sein
// - Diese Software initialisiert die CPU als Master
// - SPI-Verbindung zum Slave muss vorhanden sein (Single-Slave-System)
//     Master-PB1 <-> Slave-PB1 (SPI SCK)
//     Master-PB2 <-> Slave-PB2 (SPI MOSI)
//     Master-PB3 <-> Slave-PB3 (SPI MISO)
//     Master-PB0 <-> Slave-PB0 (SPI SS)
// - die LEDs an Port-L wechseln dauernd
//   zwischen 0x55 und 0xAA bis der Master
//   den Slave gefunden hat
// - dann wird zyklisch der Status von Button-1
//   an den Slave gesendet und an den LEDs
//   angezeigt
// - bei einem Fehler wird wieder der Slave gesucht usw.
//-----------------------------------------------------------------------------

//----------------------------------------------------
void searchSlave(void) {
  unsigned int n;
  SPI_SEND_ID_t check;
  char ready=0;
  
  while(ready==0) {
    // versucht den Slave zu finden
    
    for(n=0;n<50000;n++) {
      // kleine Pause
      MK3_LED_ZAHL(0x55);
    }
    
    check=MK3_SPIM_CHECK_SLAVE();
    if(check==SPI_NO_FAILURE) {
      // slave wurde gefunden
      ready=1;
    }
  
    for(n=0;n<50000;n++) {
      // kleine Pause
     MK3_LED_ZAHL(0xAA);
    }
  }
} 

//----------------------------------------------------
void sendSlave(void) {
  unsigned int n;
  SPI_SEND_ID_t check;
  char error=0;
  unsigned char wert=0;
  
  while(error==0) {
    // permanente Abfrage des Button-1
    // bis ein SPI-Fehler erkannt wird
    
    // Abfrage des Button-1
    if(MK3_KEY_PRESS(BUTTON1)==true) {
      // button ist gedrueckt
      wert=0xC3;
    }
    else {
      // button ist nicht gedrueckt
      wert=0;
    }
    
    // senden des Wertes an den Slave (Adr.00)
    check=MK3_SPIM_WRITE8_SLAVE(0x00,wert);
    if(check!=SPI_NO_FAILURE) error=1;
    
    // Anzeige vom Status
    for(n=0;n<50000;n++) {
      // kleine Pause
      MK3_LED_ZAHL(wert);
    }      
  }  
}

//-----------------------------------------------------------------------------
// MAIN 
//-----------------------------------------------------------------------------
int main (void)
{
    char mode=0;
    
    // Init der LEDs
	MK3_INIT_LED();
	
	// Init der Tasten
	MK3_INIT_KEY();
	
	// init der CPU als SPI-MASTER
	MK3_INIT_SPI_MASTER();
	
	while (true)
	{
		if(mode==0) {
		  // hier wird der Slave-1 gesucht
		  searchSlave();
		  // slave wurde gefunden
		  mode=1;
		}
		else {
		  // zyklisch Daten zum Slave senden
		  sendSlave();
		  // fehler beim SPI
		  mode=0;
		}
	}
	return 0;
}
//-----------------------------------------------------------------------------






