//-----------------------------------------------------------------------------
// Title      : bsp_ub034_spi_slave
//-----------------------------------------------------------------------------
// Funktion   : SPI-Kommunikation zwischen einem Master und einem Slave
// Schaltung  : myAVR MK3
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 30.10.2011
// Version    : 1.10
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <UB_MK3_2560/MK3_2560_SPI_SLAVE_SSS.h>
#include <UB_MK3_2560/MK3_2560_LED.h>

//-----------------------------------------------------------------------------
// Hinweis : 
// - Quick-Jumper Port-L muss gesetzt sein
// - Diese Software initialisiert die CPU als Slave
// - SPI-Verbindung zum Master muss vorhanden sein (Single-Slave-System)
//     Master-PB1 <-> Slave-PB1 (SPI SCK)
//     Master-PB2 <-> Slave-PB2 (SPI MOSI)
//     Master-PB3 <-> Slave-PB3 (SPI MISO)
//     Master-PB0 <-> Slave-PB0 (SPI SS)
// - die LEDs an Port-L wechseln dauernd
//   zwischen 0x0F und 0xF0 bis der Master
//   das erste Kommando an den Slave-1 gesendet hat
// - dann sendet der Master den Status von 
//   seinem Button-1 an den Slave
// - der Slave zeigt den Status an seinen LEDs an
// - Bei einem SPI-Fehler wird wieder auf den Master
//   gewartet usw.
//-----------------------------------------------------------------------------

//----------------------------------------------------
void wait4Master(void) {
  unsigned int n;
  SPI_KOM_STATUS_t check;
  char ready=0;
  
  while(ready==0) {
    // warte bis Master was gesendet hat
    
    for(n=0;n<50000;n++) {
      // kleine Pause
      MK3_LED_ZAHL(0x0F);
    }
    
    check=MK3_SPIS_AKTIVITY();
    if(check==KOM_CMD_EXE) {
      // master hat was gesendet
      ready=1;
    }
  
    for(n=0;n<50000;n++) {
      // kleine Pause
     MK3_LED_ZAHL(0xF0);
    }
  }
} 

void handleRAM(void) {
  unsigned char error=0;
  unsigned char wert;
  unsigned int n;
  SPI_KOM_STATUS_t check;

  while(error==0) {
    // permanente Abfrage vom RAM inhalt
    // bis ein SPI-Fehler erkannt wird
    
    // ram auslesen an Adr.00h
    wert=MK3_SPIS_RD8_RAM(0x00);
    
    // kleine Pause und LED anzeigen
    for(n=0;n<1000;n++) {
      // LED schalten
      if(wert==0) {
        MK3_LED_OFF(LED0);
      }
      else {
        MK3_LED_ON(LED0);
      }
    }   
    
    // test ob ein SPI-Fehler vorliegt
    check=MK3_SPIS_AKTIVITY();
    if(check==KOM_CMD_ERR) {
      // fehler ist aufgetreten
      error=1;      
    }     
  }

}



//-----------------------------------------------------------------------------
// MAIN 
//-----------------------------------------------------------------------------
int main (void)
{
	char mode=0;
	
	MK3_INIT_LED();
	
	// init der CPU als SPI-SLAVE
	MK3_INIT_SPI_SLAVE();
	
	while (true)
	{
		if(mode==0) {
			// warten auf Master
			wait4Master();
			// Master hat was gesendet
			mode=1;
		}
		else {
			// RAM bearbeiten
			handleRAM();
			// fehler beim SPI
			mode=0;
		}
	}
	return 0;
}
//-----------------------------------------------------------------------------






