//-----------------------------------------------------------------------------
// MK3_2560_KEY
// Version 1.3
// 02.11.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board
// mit ATMega-2560 @ 16MHz
// KEY-Eingabe
// Tasten an PortK Bit0 bis Bit7
// Quick-Jumper PortK muss gesetzt sein
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_KEY.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : Port-K (komplett)
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------

#define USE_MK3_KEY_PRESS 1
#define USE_MK3_KEY_SET 1
#define USE_MK3_JOY 1
#define USE_MK3_JOY_PRESS 1
#define USE_MK3_JOY_SET 1

//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_KEY" starten
// 02. Funktion "MK3_KEY_PRESS" prueft ob eine Taste gedrueckt ist
// 03. Funktion "MK3_KEY_SET" prueft ob eine Taste das 1te mal gedrueckt ist
// 10. Funktion "MK3_JOY" liest die Richtung des Joysticks aus
// 11. Funktion "MK3_JOY_PRESS" prueft ob die Richtung gewaehlt ist
// 12. Funktion "MK3_JOY_SET" prueft ob die Richtung das 1te mal gewaehlt ist
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//----------------------------------------------------------------------------- 
typedef enum {
  BUTTON1 = 0,
  BUTTON2,
  BUTTON3
}KEYTYP;

typedef enum {
  JOY_NONE = 0,
  JOY_UP,
  JOY_DOWN,
  JOY_LEFT,
  JOY_RIGHT,
  JOY_ENTER
}JOYTYP;


//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_KEY(void);	// zum initialisieren der Tasten
bool MK3_KEY_PRESS(KEYTYP key);	// check ob Taste gedrueckt
bool MK3_KEY_SET(KEYTYP key);   // check ob Taste das erste mal gedrueckt
JOYTYP MK3_JOY(void);		// check auf Joystick
bool MK3_JOY_PRESS(JOYTYP joy); // check ob Joystick gedrueckt
bool MK3_JOY_SET(JOYTYP joy);   // check ob Joystick das erste mal gedrueckt
//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------

#define MK3_DDR_KEY DDRK	// DDR-Register der Tasten
#define MK3_PORT_KEY PORTK	// Port der Tasten
#define MK3_PIN_KEY PINK	// Pin der Tasten


#define MK3_BTN1 0x01
#define MK3_BTN2 0x02
#define MK3_BTN3 0x04
#define MK3_JOYU 0x08
#define MK3_JOYL 0x10
#define MK3_JOYE 0x20
#define MK3_JOYR 0x40
#define MK3_JOYD 0x80

//-----------------------------------------------------------------------------
void MK3_INIT_KEY(void) {	// zum initialisieren der Tasten
  // alle Tasten initialisieren
  // als Eingang mit Pull-UP
  MK3_DDR_KEY = 0x00;
  MK3_PORT_KEY = 0xFF; 
}

//-----------------------------------------------------------------------------
#if USE_MK3_KEY_PRESS==1
bool MK3_KEY_PRESS(KEYTYP key) { // check ob Taste gedrueckt
  bool ret_wert=false;

  if(key==BUTTON1) {
    if((MK3_PIN_KEY&MK3_BTN1)==0) ret_wert=true;
  }
  if(key==BUTTON2) {
    if((MK3_PIN_KEY&MK3_BTN2)==0) ret_wert=true;
  }
  if(key==BUTTON3) {
    if((MK3_PIN_KEY&MK3_BTN3)==0) ret_wert=true;
  }

  return ret_wert;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_KEY_SET==1

static bool b1=false,b2=false,b3=false;

bool MK3_KEY_SET(KEYTYP key) {   // check ob Taste das erste mal gedrueckt
  bool ret_wert=false;

  if(key==BUTTON1) {
    if((MK3_PIN_KEY&MK3_BTN1)==0) {
      if(b1==false) {
        b1=true;
        ret_wert=true;
      }
    }
    else {
      b1=false;
    }
  }

  if(key==BUTTON2) {
    if((MK3_PIN_KEY&MK3_BTN2)==0) {
      if(b2==false) {
        b2=true;
        ret_wert=true;
      }
    }
    else {
      b2=false;
    }
  }

  if(key==BUTTON3) {
    if((MK3_PIN_KEY&MK3_BTN3)==0) {
      if(b3==false) {
        b3=true;
        ret_wert=true;
      }
    }
    else {
      b3=false;
    }
  }

  return ret_wert;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_JOY==1 || USE_MK3_JOY_PRESS==1 || USE_MK3_JOY_SET==1
JOYTYP MK3_JOY(void) {		// check auf Joystick
  JOYTYP ret_wert=JOY_NONE;

  if((MK3_PIN_KEY&MK3_JOYU)==0) ret_wert=JOY_UP;
  if((MK3_PIN_KEY&MK3_JOYD)==0) ret_wert=JOY_DOWN;
  if((MK3_PIN_KEY&MK3_JOYL)==0) ret_wert=JOY_LEFT;
  if((MK3_PIN_KEY&MK3_JOYR)==0) ret_wert=JOY_RIGHT;
  if((MK3_PIN_KEY&MK3_JOYE)==0) ret_wert=JOY_ENTER;

  return ret_wert;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_JOY_PRESS==1
bool MK3_JOY_PRESS(JOYTYP joy) { // check ob Joystick gedrueckt
  bool ret_wert=false;
  
  if(MK3_JOY()==joy) ret_wert=true;

  return ret_wert;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_JOY_SET==1

static JOYTYP j_old=JOY_NONE;

bool MK3_JOY_SET(JOYTYP joy) { // check ob Joystick das erste mal gedrueckt
  bool ret_wert=false;
  JOYTYP j_akt;

  j_akt=MK3_JOY();
  
  if(j_akt==joy) {
    if(j_old!=joy) {
      j_old=joy;
      ret_wert=true;
    }
  }
  else {
    j_old=j_akt;
  }

  return ret_wert;
}
#endif




