//-----------------------------------------------------------------------------
// MK3_2560_SPI_SSS
// Version 1.11
// 02.11.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board
// mit ATMega-2560 @ 16MHz
// SPI-Master-Slave definitionen (Single-Slave-System)
// SPI-Kommunikation zwischen einem Master und einem einzelnen Slave
// Hinweis : es gibt auch einen Treiber fuer ein Multi-Slave-System (MSS)
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/" liegen
//-----------------------------------------------------------------------------
// Belegt : nichts
//-----------------------------------------------------------------------------

// dieses File wird vom SPI_Master UND vom SPI_Slave eingebunden



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------



//-------------------------------------------------------------------
// Definitionen fuer alle Bytes (MSB)
// Belegt ein Nibble (wird als MSB-Nibble eingefuegt)
// zum unterscheiden aller gesendeteten bytes am MSB-Nibble
//-------------------------------------------------------------------
typedef enum {
  MSB_NC1     = 0x00,   // 0 sollte nicht benutzt werden
  MSB_BYTE_0  = 0x11,   // Kennung fuer Byte Nr. 0 vom Master (ID)
  MSB_BYTE_1  = 0x22,   // Kennung fuer Byte Nr. 1 vom Master (CMD)
  MSB_BYTE_2  = 0x33,   // Kennung fuer Byte Nr. 2 vom Master (D0_MSB)
  MSB_BYTE_3  = 0x44,   // Kennung fuer Byte Nr. 3 vom Master (D0_LSB)
  MSB_BYTE_4  = 0x55,   // Kennung fuer Byte Nr. 4 vom Master (D1_MSB)
  MSB_BYTE_5  = 0x66,   // Kennung fuer Byte Nr. 5 vom Master (D1_LSB)
  MSB_BYTE_6  = 0x77,   // Kennung fuer Byte Nr. 6 vom Master (D2_MSB)
  MSB_BYTE_7  = 0x88,   // Kennung fuer Byte Nr. 7 vom Master (D2_LSB)
  MSB_BYTE_8  = 0x99,   // Kennung fuer Byte Nr. 8 vom Master (XOR_MSB)
  MSB_BYTE_9  = 0xAA,   // Kennung fuer Byte Nr. 9 vom Master (XOR_LSB)
  MSB_FREE_3  = 0xBB,   // for future use
  MSB_FREE_2  = 0xCC,   // for future use
  MSB_FREE_1  = 0xDD,   // for future use
  MSB_SLAVE_ID= 0xEE,   // Kennung fuer alle Bytes von einem Slave
  MSB_NC2     = 0xFF    // F sollte nicht benutzt werden
}SPI_MSB_t;


//-----------------------------------------------------
// Definitionen fuer Byte Nr. 1 (CMD-Byte)
// Belegt ein Nibble (wird als LSB-Nibble eingefuegt)
// zum unterscheiden der verschiedenen Kommandos
//-----------------------------------------------------
typedef enum {
  CMD_0         = 0x00,   // Dummy-Byte (zum fuellen von nc Bytes)
  CMD_ACK       = 0x11,   // Meldung : Kommando ok und ausgefuehrt
  CMD_NACK      = 0x22,   // Meldung : fehler im Kommando
  CMD_READY     = 0x33,   // Meldung : warte auf neues Kommando
  CMD_UNKNOWN   = 0x44,   // Meldung : Kommando unbekannt
  CMD_POLL_SLAVE= 0x55,   // Kommando : liefere Daten zurueck
  CMD_CHK_SLAVE = 0x66,   // Kommando : suche Slave
  CMD_WR8_RAM   = 0x77,   // Kommando : schreibe ein byte ins RAM
  CMD_RD8_RAM   = 0x88,   // Kommando : lese ein byte aus dem RAM
  CMD_WR16_RAM  = 0x99,   // Kommando : schreibe ein INT ins RAM
  CMD_RD16_RAM  = 0xAA,   // Kommando : lese ein INT aus dem RAM
  CMD_FREE_5    = 0xBB,   // for future use
  CMD_FREE_4    = 0xCC,   // for future use
  CMD_FREE_3    = 0xDD,   // for future use  
  CMD_FREE_2    = 0xEE,   // for future use
  CMD_FREE_1    = 0xFF    // for future use
}SPI_B1_t ;

//-----------------------------------------------------
// Definitionen fuer Byte Nr. 2
// Belegt ein Nibble (wird als LSB-Nibble eingefuegt)
// als Rueckgabewert bei einem Fehler
//-----------------------------------------------------
typedef enum {
  RX_NC1        = 0x00,   // 0 sollte nicht benutzt werden  
  RX_OK         = 0x11,   // Status : Alles ok
  RX_WAIT4DATA  = 0x22,   // Status : Warte auf Daten
  RX_WRONG_XOR  = 0x33,   // Status : empfangene Checksumme ist falsch
  RX_WRONG_ID   = 0x44,   // Status : empfangene ID ist falsch
  RX_WRONG_POS  = 0x55,   // Status : empfangene Byte-Nr ist falsch
  RX_WRONG_END  = 0x66,   // Status : Ende-Kennung ist falsch
  RX_UNKNOWN    = 0x77,   // Status : Kommando unbekannt
  RX_ALL_DATA_0 = 0x88,   // Status : Alle Bits sind Lo
  RX_ALL_DATA_1 = 0x99,   // Status : Alle Bits sind Hi
  RX_WRONG_DATA = 0xAA,   // Status : Daten sind nicht erlaubt
  RX_FREE_4     = 0xBB,   // for future use
  RX_FREE_3     = 0xCC,   // for future use
  RX_FREE_2     = 0xDD,   // for future use
  RX_FREE_1     = 0xEE,   // for future use 
  RX_NC2        = 0xFF    // F sollte nicht benutzt werden    
}SPI_B2_t ;



//-----------------------------------------------------
// Einstellung der groesse des internen RAM vom Slave
// hier kann evnt. ein kleinerer Wert
// als 256 eingestellt werden
// z.B. um Speicherplatz zu sparen
//-----------------------------------------------------

#define MAX_SPI_RAM 256   // anzahl der Bytes im RAM fuer Datenaustausch
                          // max groese = 256


//-----------------------------------------------------------------------------
// FIXE Definitionen
// NICHT AENDERN !!
//-----------------------------------------------------------------------------

#define SPI_DATA_BYTES 10  // anzahl der Datenbytes die uebertragen werden
                           // Vorsicht : muss 10 sein !!

#define BPOS_ID      0     // Byte-Position fuer die ID-Kennung
#define BPOS_CMD     1     // Byte-Position fuer die CMD-Kennung
#define BPOS_XOR_HI  8     // Byte-Position fuer die XOR (Hi-Nibble)
#define BPOS_XOR_LO  9     // Byte-Position fuer die XOR (Lo-Nibble)


#define MAX_SPI_DATA_WERT SPI_DATA_BYTES*255



#define SID_SSS   0x11     // SID bei Single-Slave-System


//-----------------------------------------------------------------------------
// Struktur der SPI-Kommunikation
//-----------------------------------------------------------------------------
typedef struct {
  unsigned char tx[SPI_DATA_BYTES+1];  // absichtlich um 1 groesser
  unsigned char rx[SPI_DATA_BYTES+1];  // absichtlich um 1 groesser  
}SPI_DATA_t;
SPI_DATA_t SPI_DATA;


//-----------------------------------------------------------------------------
// erstellt aus dem Lo-Nibble von HI
// und dem Lo-Nibble von LO ein neues Byte
// 
// hi = H7,H6,H5,H4,H3,H2,H1,H0
// lo = L7,L6,L5,L4,L3,L2,L1,L0
// ret_wert <= H3,H2,H1,H0,L3,L2,L1,L0 
//-----------------------------------------------------------------------------
unsigned char SPIMS_getByte(unsigned char hi,unsigned char lo) {
  unsigned int ret_wert=0;
  unsigned char l,h;
  
  l=(lo&0x0F);
  h=(hi<<4);
  h=(h&0xF0);
  
  ret_wert=(h|l);
   
  return(ret_wert);
}

//-----------------------------------------------------------------------------
// erstellt aus dem HI-Nibble von HI
// und dem Lo-Nibble von LO ein neues Byte 
// 
// hi = H7,H6,H5,H4,H3,H2,H1,H0
// lo = L7,L6,L5,L4,L3,L2,L1,L0
// ret_wert <= H7,H6,H5,H4,L3,L2,L1,L0
//-----------------------------------------------------------------------------
unsigned char SPIMS_makeByteLo(unsigned char hi, unsigned char lo) {
  unsigned char ret_wert=0,h,l;
  
  h=(hi & 0xF0);
  l=(lo & 0x0F);
  ret_wert=(h | l);
  
  return(ret_wert);
}

//-----------------------------------------------------------------------------
// erstellt aus dem HI-Nibble von HI
// und dem Hi-Nibble von LO ein neues Byte
// 
// hi = H7,H6,H5,H4,H3,H2,H1,H0
// lo = L7,L6,L5,L4,L3,L2,L1,L0
// ret_wert <= H7,H6,H5,H4,L7,L6,L5,L4
//-----------------------------------------------------------------------------
unsigned char SPIMS_makeByteHi(unsigned char hi, unsigned char lo) {
  unsigned char ret_wert=0,h,l;
  
  h=(hi & 0xF0);
  l=(lo>>4);
  l=(l & 0x0F);
  ret_wert=(h | l);
  
  return(ret_wert);
}

//-----------------------------------------------------------------------------
// checksumme aus den TX-Daten erstellen
// XOR ueber die ersten 8 Bytes
//-----------------------------------------------------------------------------
unsigned char SPIMS_makeChecksum_TX(void) {  
  unsigned char ret_wert=0;
  unsigned int n;
  
  ret_wert=SPI_DATA.tx[0];
  for(n=1;n<8;n++) {
    ret_wert=(ret_wert ^ SPI_DATA.tx[n]);   
  }
  
  return(ret_wert);
}

//-----------------------------------------------------------------------------
// checksumme aus den RX-Daten erstellen
// XOR ueber die ersten 8 Bytes
//-----------------------------------------------------------------------------
unsigned char SPIMS_makeChecksum_RX(void) {  
  unsigned char ret_wert=0;
  unsigned int n;
  
  ret_wert=SPI_DATA.rx[0];
  for(n=1;n<8;n++) {
    ret_wert=(ret_wert ^ SPI_DATA.rx[n]);   
  }
  
  return(ret_wert);
}


